/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.novabpm.identity;

import java.security.Permission;
import java.util.Collection;
import java.util.Properties;
import java.util.Set;

import junit.framework.TestCase;

import org.ow2.novabpm.util.Misc;

/**
 * @author "Pierre Vigneras"
 * @date Nov 27, 2007
 */
public abstract class IdentityServiceOpTest extends TestCase {

  private final IdentityServiceOp service;

  public IdentityServiceOpTest() {
    this.service = newIdentityService();
  }

  protected abstract IdentityServiceOp newIdentityService();

  public void testUser() throws CommitException {
    try {
      service.getUser(Misc.getUniqueId(this.getClass().getName()));
      fail("UserNotFoundException should be thrown");
    } catch (UserNotFoundException e) {
      // ok
    }
    try {
      final String id = service.createUser();
      assertNotNull(id);
      UserOp user;
      user = service.getUser(id);
      assertNotNull(user);
      final Properties properties = user.getProperties();
      properties.setProperty("Name", "Vigneras");
      user.setProperties(properties);
      assertTrue(service.modifyUser(user));
      user = service.getUser(id);
      assertEquals("Vigneras", user.getProperties().getProperty("Name",
          "NotFound"));
      service.commit();
      try {
        service.deleteUser(Misc.getRandomString(10));
        fail("UserNotFoundException should be thrown");
      } catch (UserNotFoundException unfe) {
        // ok
      }
      service.deleteUser(id);
      try {
        service.getUser(id);
        fail("UserNotFoundException should be thrown");
      } catch (UserNotFoundException unfe) {
        // ok
      }
    } catch (UserNotFoundException unfe) {
      unfe.printStackTrace();
      fail("User not found! Should never happened unless concurrent access!");
    }
  }

  public void testGroup() throws CommitException {
    try {
      service.getGroup(Misc.getUniqueId(this.getClass().getName()));
      fail("GroupNotFoundException should be thrown");
    } catch (GroupNotFoundException e) {
      // ok
    }
    try {
      final String id = service.createGroup();
      assertNotNull(id);
      GroupOp groupOp = service.getGroup(id);
      assertNotNull(groupOp);
      final Properties properties = groupOp.getProperties();
      properties.setProperty("Name", "Admin");
      groupOp.setProperties(properties);
      assertTrue(service.modifyGroup(groupOp));
      groupOp = service.getGroup(id);
      assertEquals("Admin", groupOp.getProperties().getProperty("Name",
          "NotFound"));
      service.commit();
      try {
        service.deleteGroup(Misc.getRandomString(10));
        fail("GroupNotFoundException should be thrown");
      } catch (GroupNotFoundException gnfe) {
        // ok
      }
      service.deleteGroup(id);
      try {
        service.getGroup(id);
        fail("GroupNotFoundException should be thrown");
      } catch (GroupNotFoundException unfe) {
        // ok
      }
    } catch (GroupNotFoundException gnfe) {
      gnfe.printStackTrace();
      fail("Group not found! Should never happened unless concurrent access!");
    }
  }

  public void testGetAllUsers() {
    assertNotNull(service.getAllUsers());
  }

  public void testGetAllGroups() {
    assertNotNull(service.getAllGroups());
  }

  public void testSetMembership() throws CommitException,
      UserNotFoundException, GroupNotFoundException {
    final String userId = service.createUser();
    final UserOp userOp = service.getUser(userId);
    final String groupId = service.createGroup();
    final GroupOp groupOp = service.getGroup(groupId);

    assertFalse(userOp.isMember(groupOp));
    Set<Membership> memberships = userOp.getMemberships();
    for (Membership membership : memberships) {
      assertFalse(membership.getGroup().equals(groupOp));
    }
    assertFalse(groupOp.hasMember(userOp));
    memberships = groupOp.getMemberships();
    for (Membership membership : memberships) {
      assertFalse(membership.getUser().equals(userOp));
    }
    final Membership membership = service.setMembership(userOp, groupOp);
    assertTrue(userOp.isMember(groupOp));
    assertTrue(userOp.getMemberships().contains(membership));
    assertTrue(groupOp.getMemberships().contains(membership));
    service.commit();
  }

  protected static final int USER_MIN_NB_IN_GROUP = 0;

  protected static final int USER_MAX_NB_IN_GROUP = 10;

  protected static final int MIN_GROUP_NB = 5;

  protected static final int MAX_GROUP_NB = 10;

  public static int populate(final IdentityServiceOp service, final int groupNb)
      throws UserNotFoundException, GroupNotFoundException {
    int userNb = 0;
    int gNb = groupNb;
    GroupOp root = null;
    GroupOp parent = null;
    for (; gNb > 0; gNb--) {
      // Build a RANDOM hierarchy of groupOps
      String groupId;
      GroupOp groupOp;
      if (root == null) { // First time!
        groupId = service.createGroup();
        root = service.getGroup(groupId);
        parent = root;
        groupOp = root;
      } else {
        if (Misc.RANDOM.nextBoolean())
          parent = root; // Create the group from root
        groupId = service.createGroup(parent);
        groupOp = service.getGroup(groupId);
        parent = groupOp;
      }
      Properties properties = groupOp.getProperties();
      fillProperties(groupId, properties);
      groupOp.setProperties(properties);
      Set<Permission> permissions = groupOp.getPermissions();
      fillPermissions(permissions);
      groupOp.setPermissions(permissions);
      service.modifyGroup(groupOp);
      int uNb = USER_MIN_NB_IN_GROUP
          + Misc.RANDOM.nextInt(USER_MAX_NB_IN_GROUP - USER_MIN_NB_IN_GROUP);
      userNb += uNb;
      for (; uNb > 0; uNb--) {
        final String userId = service.createUser();
        final UserOp userOp = service.getUser(userId);
        service.setMembership(userOp, groupOp);
        properties = userOp.getProperties();
        fillProperties(userId, properties);
        userOp.setProperties(properties);
        permissions = userOp.getPermissions();
        fillPermissions(permissions);
        userOp.setPermissions(permissions);
      }
    }
    return userNb;
  }

  private static void fillProperties(String groupId, Properties properties) {
    int pNb = 1 + Misc.RANDOM.nextInt(5);
    while (pNb-- > 0) {
      properties.setProperty(groupId + "-PropertyKey#" + pNb, groupId
          + "-PropertyValue#" + pNb);
    }
  }

  private static void fillPermissions(Set<Permission> permissions) {
    int pNb = 1 + Misc.RANDOM.nextInt(5);
    while (pNb-- > 0) {
      permissions.add(new DummyPermission(Misc.getRandomString(10)));
    }
  }

  public void testPopulate() throws CommitException, UserNotFoundException,
      GroupNotFoundException {
    final int groupNb = MIN_GROUP_NB
        + Misc.RANDOM.nextInt(MAX_GROUP_NB - MIN_GROUP_NB);
    final int userNb = IdentityServiceOpTest.populate(service, groupNb);
    service.commit();
    assertTrue(service.getAllUsers().size() == userNb);
    assertTrue(service.getAllGroups().size() == groupNb);
    IdentityServiceOpTest.checkConsistency(service.getAllGroups(), service
        .getAllUsers());
  }

  public static void checkConsistency(Collection<? extends GroupOp> groupOps,
      Collection<? extends UserOp> users) {
    // First, check the link user -> group
    for (UserOp u : users) {
      final Set<Membership> memberships = u.getMemberships();
      for (Membership m : memberships) {
        assertTrue(groupOps.contains(m.getGroup()));
      }
    }
    // Second, check the link group -> user
    for (GroupOp g : groupOps) {
      final Set<Membership> memberships = g.getMemberships();
      for (Membership m : memberships) {
        assertTrue(users.contains(m.getUser()));
      }
    }
    // Third, check the GroupOp tree
    GroupOp root = null;
    for (GroupOp g : groupOps) {
      if (root == null)
        root = g.getRoot();
      assertEquals("Root of " + g + " should be " + root, root, g.getRoot());
      final Set<GroupOp> children = g.getChildren();
      if (!children.isEmpty()) {
        for (GroupOp c : children) {
          assertEquals("Parent of child " + c.getId() + " should be "
              + g.getId(), g, c.getParent());
        }
      }
    }
  }
}
